@@ -74,6 +74,9 @@ class PredictionContext(object):
     def __init__(self, cachedHashCode:int):
         self.cachedHashCode = cachedHashCode
 
+    def __len__(self):
+        return 0
+
     # This means only the {@link #EMPTY} context is in set.
     def isEmpty(self):
         return self is self.EMPTY
@@ -81,16 +84,20 @@ class PredictionContext(object):
     def hasEmptyPath(self):
         return self.getReturnState(len(self) - 1) == self.EMPTY_RETURN_STATE
 
+    def getReturnState(self, index:int):
+        raise "illegal!"
+
     def __hash__(self):
         return self.cachedHashCode
 
-
 def calculateHashCode(parent:PredictionContext, returnState:int):
-    return hash( str(parent) + str(returnState))
-
-def calculateEmptyHashCode():
-    return hash("")
+    return hash("") if parent is None else hash((hash(parent), returnState))
 
+def calculateListsHashCode(parents:[], returnStates:[] ):
+    h = 0
+    for parent, returnState in parents, returnStates:
+        h = hash((h, calculateHashCode(parent, returnState)))
+    return h
 
 #  Used to cache {@link PredictionContext} objects. Its used for the shared
 #  context cash associated with contexts in DFA states. This cache
@@ -132,8 +139,7 @@ class SingletonPredictionContext(PredictionContext):
             return SingletonPredictionContext(parent, returnState)
 
     def __init__(self, parent:PredictionContext, returnState:int):
-        assert returnState!=ATNState.INVALID_STATE_NUMBER
-        hashCode = calculateHashCode(parent, returnState) if parent is not None else calculateEmptyHashCode()
+        hashCode = calculateHashCode(parent, returnState)
         super().__init__(hashCode)
         self.parentCtx = parent
         self.returnState = returnState
@@ -142,11 +148,9 @@ class SingletonPredictionContext(PredictionContext):
         return 1
 
     def getParent(self, index:int):
-        assert index == 0
         return self.parentCtx
 
     def getReturnState(self, index:int):
-        assert index == 0
         return self.returnState
 
     def __eq__(self, other):
@@ -156,10 +160,8 @@ class SingletonPredictionContext(PredictionContext):
             return False
         elif not isinstance(other, SingletonPredictionContext):
             return False
-        elif hash(self) != hash(other):
-            return False # can't be same if hash is different
         else:
-            return self.returnState == other.returnState and self.parentCtx==other.parentCtx
+            return self.returnState == other.returnState and self.parentCtx == other.parentCtx
 
     def __hash__(self):
         return self.cachedHashCode
@@ -183,15 +185,12 @@ class EmptyPredictionContext(SingletonPredictionContext):
     def isEmpty(self):
         return True
 
-    def getParent(self, index:int):
-        return None
-
-    def getReturnState(self, index:int):
-        return self.returnState
-
     def __eq__(self, other):
         return self is other
 
+    def __hash__(self):
+        return self.cachedHashCode
+
     def __str__(self):
         return "$"
 
@@ -204,9 +203,7 @@ class ArrayPredictionContext(PredictionContext):
     #  returnState == {@link #EMPTY_RETURN_STATE}.
 
     def __init__(self, parents:list, returnStates:list):
-        super().__init__(calculateHashCode(parents, returnStates))
-        assert parents is not None and len(parents)>0
-        assert returnStates is not None and len(returnStates)>0
+        super().__init__(calculateListsHashCode(parents, returnStates))
         self.parents = parents
         self.returnStates = returnStates
 
@@ -274,17 +271,7 @@ def PredictionContextFromRuleContext(atn:ATN, outerContext:RuleContext=None):
     return SingletonPredictionContext.create(parent, transition.followState.stateNumber)
 
 
-def calculateListsHashCode(parents:[], returnStates:int ):
-
-    with StringIO() as s:
-        for parent in parents:
-            s.write(str(parent))
-        for returnState in returnStates:
-            s.write(str(returnState))
-        return hash(s.getvalue())
-
 def merge(a:PredictionContext, b:PredictionContext, rootIsWildcard:bool, mergeCache:dict):
-    assert a is not None and b is not None # must be empty context, never null
 
     # share same graph if both same
     if a==b:
@@ -303,9 +290,9 @@ def merge(a:PredictionContext, b:PredictionContext, rootIsWildcard:bool, mergeCa
 
     # convert singleton so both are arrays to normalize
     if isinstance( a, SingletonPredictionContext ):
-        a = ArrayPredictionContext([a.parent], [a.returnState])
+        a = ArrayPredictionContext([a.parentCtx], [a.returnState])
     if isinstance( b, SingletonPredictionContext):
-        b = ArrayPredictionContext([b.parent], [b.returnState])
+        b = ArrayPredictionContext([b.parentCtx], [b.returnState])
     return mergeArrays(a, b, rootIsWildcard, mergeCache)
 
 
