@@ -56,9 +56,9 @@ class DFASerializer(object):
                     if t is not None and t.stateNumber != 0x7FFFFFFF:
                         buf.write(self.getStateString(s))
                         label = self.getEdgeLabel(i)
-                        buf.write(u"-")
+                        buf.write("-")
                         buf.write(label)
-                        buf.write(u"->")
+                        buf.write("->")
                         buf.write(self.getStateString(t))
                         buf.write('\n')
             output = buf.getvalue()
